// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot;

import frc.robot.constants.OIConstants;
import frc.robot.subsystems.Arm;
import frc.robot.subsystems.ClimberPivot;
import frc.robot.subsystems.ClimberRollers;
import frc.robot.subsystems.Drivetrain;
import frc.robot.subsystems.Elevator;
import frc.robot.subsystems.Indexer;
import frc.robot.subsystems.Manipulator;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.PrintCommand;
import edu.wpi.first.wpilibj2.command.button.CommandXboxController;

public class RobotContainer {
	private Arm arm;

	private ClimberPivot climberPivot;

	private ClimberRollers climberRollers;

	private Drivetrain drivetrain;

	private Elevator elevator;

	private Indexer indexer;

	private Manipulator manipulator;

	private CommandXboxController driver;
	private CommandXboxController operator;

	public RobotContainer() {
		arm = new Arm();

		climberPivot = new ClimberPivot();

		climberRollers = new ClimberRollers();

		drivetrain = new Drivetrain();

		elevator = new Elevator();

		indexer = new Indexer();

		manipulator = new Manipulator();

		driver = new CommandXboxController(OIConstants.kDriverControllerPort);
		operator = new CommandXboxController(OIConstants.kOperatorControllerPort);

		configureButtonBindings();
	}

	private void configureButtonBindings() {
		arm.setDefaultCommand(
			arm.goToSetpoint(0, 1)
		);

		climberPivot.setDefaultCommand(
			climberPivot.goToAngle(0)
		);

		climberRollers.setDefaultCommand(
			climberRollers.runRoller(0)
		);

		drivetrain.setDefaultCommand(
			drivetrain.drive(
				driver::getLeftY,
				driver::getLeftX,
				driver::getRightX,
				() -> true
			)
		);

		elevator.setDefaultCommand(
			elevator.goToSetpoint(0, 1)
		);

		indexer.setDefaultCommand(
			indexer.runIndexer(0)
		);

		manipulator.setDefaultCommand(
			manipulator.runManipulator(0)
		);

		//Driver inputs
		driver.start().whileTrue(
			drivetrain.setXCommand()
		);

		driver.rightTrigger().whileTrue(
			manipulator.runManipulator(1)
		);

		//Operator inputs
		operator.povUp().onTrue(
			elevator.goToSetpoint(0, 0)
		);

		operator.povRight().onTrue(
			elevator.goToSetpoint(0, 0)
		);

		operator.povLeft().onTrue(
			elevator.goToSetpoint(0, 0)
		);

		operator.povDown().onTrue(
			elevator.goToSetpoint(0, 0)
		);
	}

	public Command getAutonomousCommand() {
		return new PrintCommand("NO AUTO DEFINED");
	}
}
