package frc.robot.subsystems;

import com.ctre.phoenix6.hardware.CANcoder;
import com.revrobotics.spark.SparkMax;
import com.revrobotics.spark.SparkLowLevel.MotorType;

import edu.wpi.first.math.controller.ArmFeedforward;
import edu.wpi.first.math.controller.PIDController;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.ArmConstants;

public class Arm extends SubsystemBase {
    private SparkMax ArmMotor;

    private CANcoder canCoder;

    private PIDController pidController;
    private ArmFeedforward feedForward;

    public Arm() {
        ArmMotor = new SparkMax(
            ArmConstants.kArmMotorID, 
            MotorType.kBrushless
        );

        canCoder = new CANcoder(ArmConstants.kCANcoderID);
    }

    public Command goToSetpoint(double setpoint, double timeout) {
        return run(() -> {
            double voltsOut = pidController.calculate(
                canCoder.getPosition().getValueAsDouble(), 
                setpoint
            ) + feedForward.calculate(
                canCoder.getPosition().getValueAsDouble(), 
                canCoder.getVelocity().getValueAsDouble()
            );

            ArmMotor.setVoltage(voltsOut);
        }).until(pidController::atSetpoint).withTimeout(timeout);
    }
}
