package frc.robot.subsystems;

import com.revrobotics.AbsoluteEncoder;
import com.revrobotics.spark.SparkMax;
import com.revrobotics.spark.SparkLowLevel.MotorType;

import edu.wpi.first.math.controller.PIDController;
import edu.wpi.first.wpilibj.DigitalInput;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.ClimberPivotConstants;

public class ClimberPivot extends SubsystemBase {
    private SparkMax pivotMotor;

    private AbsoluteEncoder neoEncoder;

    private DigitalInput cageLimitSwitch;

    private PIDController pidController;

    public ClimberPivot() {
        pivotMotor = new SparkMax(
            ClimberPivotConstants.kPivotMotorID, 
            MotorType.kBrushless
        );

        neoEncoder = pivotMotor.getAbsoluteEncoder();

        cageLimitSwitch = new DigitalInput(0);

        pidController = new PIDController(
            ClimberPivotConstants.kPIDControllerP,
            ClimberPivotConstants.kPIDControllerI, 
            ClimberPivotConstants.kPIDControllerD
        );
    }

    public Command runPivot(double speed) {
        return run(() -> {
            pivotMotor.set(speed);
        });
    }

    public Command goToAngle(double setpoint) {
        return run(() -> {
            pivotMotor.set(
                pidController.calculate(
                    neoEncoder.getPosition(), 
                    setpoint
                )
            );
        });
    }

    public boolean getCageLimitSwitch() {
        return cageLimitSwitch.get();
    }
}
