package frc.robot.subsystems;

import com.revrobotics.RelativeEncoder;
import com.revrobotics.spark.SparkMax;
import com.revrobotics.spark.SparkBase.PersistMode;
import com.revrobotics.spark.SparkBase.ResetMode;
import com.revrobotics.spark.SparkLowLevel.MotorType;

import edu.wpi.first.math.controller.ElevatorFeedforward;
import edu.wpi.first.math.controller.PIDController;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.ElevatorConstants;

public class Elevator extends SubsystemBase {
    protected SparkMax elevatorMotor1;

    protected RelativeEncoder encoder;

    private PIDController pidController;

    private ElevatorFeedforward feedForward;
    
    public Elevator() {
        elevatorMotor1 = new SparkMax(
            ElevatorConstants.kElevatorMotorID, 
            MotorType.kBrushless
        );

        elevatorMotor1.configure(
            ElevatorConstants.motorConfig, 
            ResetMode.kResetSafeParameters, 
            PersistMode.kPersistParameters
        );

        encoder = elevatorMotor1.getEncoder();

        pidController = new PIDController(
            ElevatorConstants.kPIDControllerP, 
            ElevatorConstants.kPIDControllerI, 
            ElevatorConstants.kPIDControllerD
        );

        feedForward = new ElevatorFeedforward(
            ElevatorConstants.kFeedForwardS, 
            ElevatorConstants.kFeedForwardG, 
            ElevatorConstants.kFeedForwardV
        );
    }

    public Command runElevator(double speed) {
        return run(() -> {
            elevatorMotor1.set(speed);
        });
    }
    
    public Command goToSetpoint(double setpoint, double timeout) {
        return run(() -> {
            double voltsOut = pidController.calculate(
                encoder.getPosition(),
                setpoint
            ) + feedForward.calculate(0);

            elevatorMotor1.setVoltage(voltsOut);
        }).until(pidController::atSetpoint).withTimeout(timeout);
    }
}
